/* This file was generated by syncqt. */
#ifndef QT_QTGLXSUPPORT_VERSION_H
#define QT_QTGLXSUPPORT_VERSION_H

#define QTGLXSUPPORT_VERSION_STR "5.15.19"

#define QTGLXSUPPORT_VERSION 0x050F13

#endif // QT_QTGLXSUPPORT_VERSION_H
