/* This file was generated by syncqt. */
#ifndef QT_QTTHEMESUPPORT_VERSION_H
#define QT_QTTHEMESUPPORT_VERSION_H

#define QTTHEMESUPPORT_VERSION_STR "5.15.19"

#define QTTHEMESUPPORT_VERSION 0x050F13

#endif // QT_QTTHEMESUPPORT_VERSION_H
